@@ -20,9 +20,7 @@
 //    GNU Lesser General Public License for more details.
 //
 //    You should have received a copy of the GNU Lesser General Public
-//    License along with this library; if not, write to the Free
-//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-//    MA 02111-1307, USA
+//    License along with this library. If not, see http://www.gnu.org/licenses/
 //
 //
 // Description:
@@ -49,6 +47,7 @@ PyInterpreterState* omniPy::pyInterpreter;
 PyObject* omniPy::py_omnipymodule;	// The _omnipy extension
 PyObject* omniPy::py_pseudoFns;         //  pseudoFns
 PyObject* omniPy::py_policyFns;         //  policyFns
+PyObject* omniPy::py_callInfoFns;       //  callInfoFns
 PyObject* omniPy::pyCORBAmodule;	// The CORBA module
 PyObject* omniPy::pyCORBAsysExcMap;	//  The system exception map
 PyObject* omniPy::pyCORBAORBClass;	//  ORB class
@@ -377,7 +376,7 @@ extern "C" {
 	delete[] argv;
 	return 0;
       }
-      argv[i] = String_AsString(o);
+      argv[i] = (char*)String_AsString(o);
     }
 
     int orig_argc = argc;
@@ -404,7 +403,7 @@ extern "C" {
 
 	while (1) {
 	  o = PyList_GetItem(pyargv, i); OMNIORB_ASSERT(o != 0);
-	  t = String_AsString(o);
+	  t = (char*)String_AsString(o);
 	  if (s == t) break;
 	  r = PySequence_DelItem(pyargv, i);
 	  OMNIORB_ASSERT(r != -1);
@@ -629,8 +628,9 @@ extern "C" {
   {
     PyObject* d = PyModule_GetDict(m);
 
-    PyDict_SetItemString(d, (char*)"__version__",
-			 String_FromString(OMNIPY_VERSION_STRING));
+    PyObject* ver = String_FromString(OMNIPY_VERSION_STRING);
+    PyDict_SetItemString(d, (char*)"__version__", ver);
+    Py_DECREF(ver);
 
     PyObject* excs = generateExceptionList();
     PyDict_SetItemString(d, (char*)"system_exceptions", excs);
@@ -669,6 +669,12 @@ extern "C" {
     PyDict_SetItemString(d, (char*)"policyFns", omniPy::py_policyFns);
     Py_DECREF(omniPy::py_policyFns);
 
+    // Empty dict for external transport modules to register
+    // additional call info functions.
+    omniPy::py_callInfoFns = PyDict_New();
+    PyDict_SetItemString(d, (char*)"callInfoFns", omniPy::py_callInfoFns);
+    Py_DECREF(omniPy::py_callInfoFns);
+
     // Codesets
     omniPy::ncs_c_utf_8 = omniCodeSet::getNCS_C("UTF-8");
 
@@ -704,8 +710,10 @@ extern "C" {
   PyMODINIT_FUNC
   PyInit__omnipy(void)
   {
+#if (PY_VERSION_HEX < 0x03070000)
     // Make sure Python is running multi-threaded
     PyEval_InitThreads();
+#endif
 
     PyObject* m = PyModule_Create(&omnipymodule);
     if (!m)
