@@ -20,9 +20,7 @@
 //    GNU Lesser General Public License for more details.
 //
 //    You should have received a copy of the GNU Lesser General Public
-//    License along with this library; if not, write to the Free
-//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-//    MA 02111-1307, USA
+//    License along with this library. If not, see http://www.gnu.org/licenses/
 //
 //
 // Description:
@@ -218,6 +216,7 @@ public:
   static PyObject* py_omnipymodule;    	// _omnipy module
   static PyObject* py_pseudoFns;        //  pseudoFns
   static PyObject* py_policyFns;        //  policyFns
+  static PyObject* py_callInfoFns;      //  calInfoFns
   static PyObject* pyCORBAmodule;      	// CORBA module
   static PyObject* pyCORBAsysExcMap;   	//  The system exception map
   static PyObject* pyCORBAORBClass;    	//  ORB class
@@ -300,7 +299,6 @@ public:
   static void initFixed          (PyObject* d);
   static void initCallDescriptor (PyObject* d);
   static void initServant        (PyObject* d);
-  static void initTypeCode       (PyObject* d);
 
 
   ////////////////////////////////////////////////////////////////////////////
@@ -379,6 +377,11 @@ public:
     // Pointer operator used in some Python macros like PyInt_Check.
     inline PyObject* operator->()      { return obj_; }
 
+#ifdef PYPY_VERSION
+    // PyPy defines macros taking a void* rather than a PyObject*
+    inline operator void*()            { return obj_; }
+#endif
+
   private:
     PyObject* obj_;
 
@@ -468,7 +471,7 @@ public:
       long v = PyInt_AS_LONG(obj);
 
       if (v < 0
-#if SIZEOF_LONG > 4
+#if OMNI_SIZEOF_LONG > 4
           || v > 0xffffffff
 #endif
           ) {
@@ -487,7 +490,7 @@ public:
     unsigned long v = PyLong_AsUnsignedLong(obj);
 
     if (PyErr_Occurred() 
-#if SIZEOF_LONG > 4
+#if OMNI_SIZEOF_LONG > 4
         || v > 0xffffffff
 #endif
         ) {
@@ -1492,7 +1495,7 @@ public:
 
 };
 
-#ifdef HAS_Cplusplus_catch_exception_by_base
+#ifdef OMNI_HAS_Cplusplus_catch_exception_by_base
 
 #define OMNIPY_CATCH_AND_HANDLE_SYSTEM_EXCEPTIONS \
 catch (Py_BAD_PARAM& ex) { \
