@@ -20,9 +20,7 @@
 //    GNU Lesser General Public License for more details.
 //
 //    You should have received a copy of the GNU Lesser General Public
-//    License along with this library; if not, write to the Free
-//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-//    MA 02111-1307, USA
+//    License along with this library. If not, see http://www.gnu.org/licenses/
 //
 //
 // Description:
@@ -32,7 +30,7 @@
 #include <pyThreadCache.h>
 #include <omniORB4/IOP_C.h>
 
-#ifdef HAS_Cplusplus_Namespace
+#ifdef OMNI_HAS_Cplusplus_Namespace
 namespace {
 #endif
   class cdLockHolder {
@@ -55,7 +53,7 @@ namespace {
     omniPy::InterpreterUnlocker*  ul_;
     omnipyThreadCache::CacheNode* cn_;
   };
-#ifdef HAS_Cplusplus_Namespace
+#ifdef OMNI_HAS_Cplusplus_Namespace
 };
 #endif
 
