@@ -20,9 +20,7 @@
 //    GNU Lesser General Public License for more details.
 //
 //    You should have received a copy of the GNU Lesser General Public
-//    License along with this library; if not, write to the Free
-//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-//    MA 02111-1307, USA
+//    License along with this library. If not, see http://www.gnu.org/licenses/
 //
 //
 // Description:
@@ -164,6 +162,12 @@ createPolicyObject(PortableServer::POA_ptr poa, PyObject* pypolicy)
       }
       break;
 
+    case 0x41545403: // PlainObjectKeysPolicy
+      {
+        policy = new omniPolicy::PlainObjectKeysPolicy(PyObject_IsTrue(pyvalue));
+      }
+      break;
+
     default:
       {
         // Is there a function registered in _omnipy.policyFns?
