@@ -20,9 +20,7 @@
 //    GNU Lesser General Public License for more details.
 //
 //    You should have received a copy of the GNU Lesser General Public
-//    License along with this library; if not, write to the Free
-//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-//    MA 02111-1307, USA
+//    License along with this library. If not, see http://www.gnu.org/licenses/
 //
 //
 // Description:
@@ -36,7 +34,9 @@ static int static_cleanup = 0;
 // Set true when static data is being destroyed. Used to make sure
 // Python things aren't used after they have gone away.
 
-omni_mutex*                    omnipyThreadCache::guard      = 0;
+static omni_mutex the_guard;
+
+omni_mutex*                    omnipyThreadCache::guard      = &the_guard;
 const unsigned int             omnipyThreadCache::tableSize  = 67;
 omnipyThreadCache::CacheNode** omnipyThreadCache::table      = 0;
 unsigned int                   omnipyThreadCache::scanPeriod = 30;
@@ -92,7 +92,6 @@ omnipyThreadCache::
 init()
 {
   omnithread_key = omni_thread::allocate_key();
-  guard          = new omni_mutex();
   table          = new CacheNode*[tableSize];
   for (unsigned int i=0; i < tableSize; i++) table[i] = 0;
 
@@ -107,9 +106,7 @@ shutdown()
   if (the_scavenger) the_scavenger->kill();
   the_scavenger = 0;
 
-  if (guard) delete guard;
   table = 0;
-  guard = 0;
 }
 
 
@@ -185,8 +182,8 @@ addNewNode(long id, unsigned int hash)
   // that case, the re-entry will get a valid Python thread state,
   // albeit without a threading.Thread object.
 
-  cn->workerThread = PyEval_CallObject(omniPy::pyWorkerThreadClass,
-				       omniPy::pyEmptyTuple);
+  cn->workerThread = PyObject_CallObject(omniPy::pyWorkerThreadClass,
+				         omniPy::pyEmptyTuple);
   if (!cn->workerThread) {
     if (omniORB::trace(1)) {
       {
@@ -287,8 +284,8 @@ run_undetached(void*)
   PyThreadState_Swap(threadState_);
 #endif
 
-  workerThread_ = PyEval_CallObject(omniPy::pyWorkerThreadClass,
-				    omniPy::pyEmptyTuple);
+  workerThread_ = PyObject_CallObject(omniPy::pyWorkerThreadClass,
+				      omniPy::pyEmptyTuple);
   if (!workerThread_) {
     if (omniORB::trace(2)) {
       omniORB::logs(2, "Exception trying to create WorkerThread for thread "
@@ -461,9 +458,9 @@ run_undetached(void*)
 }
 
 
-class _omnipy_cleapup_detector {
+class _omnipy_cleanup_detector {
 public:
-  inline ~_omnipy_cleapup_detector() { static_cleanup = 1; }
+  inline ~_omnipy_cleanup_detector() { static_cleanup = 1; }
 };
 
-static _omnipy_cleapup_detector _the_omnipy_cleapup_detector;
+static _omnipy_cleanup_detector _the_omnipy_cleanup_detector;
