@@ -8,19 +8,17 @@
 //    This file is part of the omniORB library
 //
 //    The omniORB library is free software; you can redistribute it and/or
-//    modify it under the terms of the GNU Library General Public
+//    modify it under the terms of the GNU Lesser General Public
 //    License as published by the Free Software Foundation; either
-//    version 2 of the License, or (at your option) any later version.
+//    version 2.1 of the License, or (at your option) any later version.
 //
 //    This library is distributed in the hope that it will be useful,
 //    but WITHOUT ANY WARRANTY; without even the implied warranty of
 //    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-//    Library General Public License for more details.
+//    Lesser General Public License for more details.
 //
-//    You should have received a copy of the GNU Library General Public
-//    License along with this library; if not, write to the Free
-//    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-//    02111-1307, USA
+//    You should have received a copy of the GNU Lesser General Public
+//    License along with this library. If not, see http://www.gnu.org/licenses/
 //
 // Description:
 //    ZIOP support
@@ -89,7 +87,7 @@ getULong(PyObject* obj)
                     BAD_PARAM_PythonValueOutOfRange,
                     CORBA::COMPLETED_NO);
     }
-#if SIZEOF_LONG > 4
+#if OMNI_SIZEOF_LONG > 4
     if (ul > 0xffffffffL) {
       OMNIORB_THROW(BAD_PARAM,
                     BAD_PARAM_PythonValueOutOfRange,
@@ -101,7 +99,7 @@ getULong(PyObject* obj)
 #if (PY_VERSION_HEX < 0x03000000)
   else if (PyInt_Check(obj)) {
     long l = PyInt_AS_LONG(obj);
-#if SIZEOF_LONG > 4
+#if OMNI_SIZEOF_LONG > 4
     if (l < 0 || l > 0xffffffffL) {
       OMNIORB_THROW(BAD_PARAM,
                     BAD_PARAM_PythonValueOutOfRange,
